<?php
	/**
	 * @package     Joomla.Administrator
	 * @subpackage  com_formea
	 *
	 * @copyright   Copyright (C) 2010-2024 Feseur Sdn Bhd. All rights reserved.
	 * @license     GNU General Public License version 2 or later; see LICENSE.txt
	 */

	namespace Joomla\Component\Formea\Administrator\Model;

	defined('_JEXEC') or die;

	use Joomla\Database\DatabaseQuery;
	use Joomla\CMS\Application\CMSApplicationInterface;
	use Joomla\CMS\Factory;
	use Joomla\CMS\MVC\Model\ListModel;
	use RuntimeException;

	/**
	 * Methods supporting a list of formea records.
	 *
	 * @since  1.0
	 */
	class FormeasModel extends ListModel
	{
		/**
		 * @var CMSApplicationInterface|null
		 * @since 1.0.0
		 */
		protected $app;

		/**
		 * Constructor.
		 *
		 * @param   array  $config  An optional associative array of configuration settings.
		 *
		 * @see     \JControllerLegacy
		 * @since   1.0
		 */
		public function __construct($config = [])
		{

			if (empty($config['filter_fields']))
			{
				$config['filter_fields'] = [
					'id', 'a.id',
					'state', 'a.state',
					'access_level', 'a.access_level',
					'checked_out', 'a.checked_out',
					'checked_out_time', 'a.checked_out_time',
					'publish_up', 'a.publish_up',
					'publish_down', 'a.publish_down',
					'created_date', 'a.created_date',
					'created_by', 'a.created_by',
					'modified_date', 'a.modified_date',
					'modified_by', 'a.modified_by',
					'title', 'a.title',
					'alia', 'a.alias',
				];
			}

			parent::__construct($config);
			$this->app = Factory::getApplication();

		}

		/**
		 * Method to get a DatabaseQuery object for retrieving the data set from a database.
		 *
		 * @return  DatabaseQuery|string  A DatabaseQuery object to retrieve the data set.
		 *
		 * @since   1.6
		 */
		protected function getListQuery()
		{
			$columns = [
				'a.id',
				'a.state',
				'a.access_level',
				'a.checked_out',
				'a.checked_out_time',
				'a.publish_up',
				'a.publish_down',
				'a.created_date',
				'a.created_by',
				'a.modified_date',
				'a.modified_by',
				'a.title',
				'a.alias'
			];
			// Create a new query object.
			$db    = $this->getDatabase();
			$query = $db->getQuery(true);

			// Select the required fields from the table.
			$query->select(
				$db->quoteName(
					explode(
						', ',
						$this->getState(
							'list.select', implode(', ', $columns))
					)
				)
			);

			$query->from($db->quoteName('#__formea_forms', 'a'));
			$query->select($db->quoteName('b.title', 'access_level_title'))->innerJoin(
				$db->quoteName('#__viewlevels', 'b') . ' ON ' . $db->quoteName('a.access_level') . ' = ' .
				$db->quoteName('b.id')
			);


			// Join over the users for the checked out user.
			$query->select($db->quoteName('uc.name', 'editor'))
				->join(
					'LEFT',
					$db->quoteName('#__users', 'uc') . ' ON ' . $db->quoteName('uc.id') . ' = ' . $db->quoteName('a.checked_out')
				);


			// Filter by access level.
			if ($access = $this->getState('filter.access'))
			{
				$query->where($db->quoteName('a.access_level') . ' = ' . (int) $access);
			}

			// Filter by published state
			$published = (string) $this->getState('filter.published');

			if (is_numeric($published))
			{
				$query->where($db->quoteName('a.state') . ' = ' . (int) $published);
			}
			elseif ($published === '')
			{
				$query->where($db->quoteName('a.state') . ' = 0');
				$query->extendWhere('OR', $db->quoteName('a.state') . ' = 1');
			}
			$subQuery = $db->getQuery(true);
			$subQuery->select('COUNT(*)')
				->from($db->quoteName('#__formea_submissions', 'sb'))
				->where($db->quoteName('a.id') . ' = ' . $db->quoteName('sb.form_id'));
			$query->select('(' . $subQuery . ') AS submissions');

			// Filter by search in name.
			$search = $this->getState('filter.search');

			if (!empty($search))
			{
				if (stripos($search, 'id:') === 0)
				{
					$query->where('a.id = ' . (int) substr($search, 3));
				}
				else
				{
					$search = $db->quote('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
					$query->where(
						'(' . $db->quoteName('a.name') . ' LIKE ' . $search . ')'
					);
				}
			}

			// Filter on the language.
			if ($language = $this->getState('filter.language'))
			{
				$query->where($db->quoteName('a.language') . ' = ' . $db->quote($language));
			}

			// Add the list ordering clause.
			$orderCol  = $this->state->get('list.ordering', 'a.name');
			$orderDirn = $this->state->get('list.direction', 'asc');

			if ($orderCol == 'a.ordering' || $orderCol == 'category_title')
			{
				$orderCol = $db->quoteName('c.title') . ' ' . $orderDirn . ', ' . $db->quoteName('a.ordering');
			}

			$query->order($db->escape($orderCol . ' ' . $orderDirn));

			return $query;
		}

		/**
		 * Method to auto-populate the model state.
		 *
		 * Note. Calling getState in this method will result in recursion.
		 *
		 * @param   string  $ordering   An optional ordering field.
		 * @param   string  $direction  An optional direction (asc|desc).
		 *
		 * @return  void
		 *
		 * @since   1.0
		 */
		protected function populateState($ordering = 'a.id', $direction = 'desc')
		{
			$app = Factory::getApplication();

			// Adjust the context to support modal layouts.
			if ($layout = $app->input->get('layout'))
			{
				$this->context .= '.' . $layout;
			}

			// List state information.
			parent::populateState($ordering, $direction);
		}

		public function getItems()
		{
			// Get a storage key.
			$store = $this->getStoreId();

			// Try to load the data from internal storage.
			if (isset($this->cache[$store]))
			{
				return $this->cache[$store];
			}

			try
			{
				// Load the list items and add the items to the internal cache.
				$this->cache[$store] = $this->_getList($this->_getListQuery(), $this->getStart(), $this->getState('list.limit'));
			}
			catch (RuntimeException $e)
			{
				$this->app->enqueueMessage($e->getMessage(), 'error');

				return false;
			}

			return $this->cache[$store];
		}
	}
